#include "common.h"
#include "result.h"
#include "tdrocr.h"

int isNewCamera();
int isOldCamera();
extern OCR_RESULT card_result;
extern cv::Mat g_cropimg;
extern cv::Mat g_faceimg;
// extern OCR_RESULT ocr_result;
int check_and_rotated_card_front(Mat& src, Rect rectFace, vector<Rect>& linesSelect, int cardClass, int camera);
int checkFrontValid(cv::Mat & card, int camera);
void draw_text_allboxs(const cv::Mat& img, std::vector<RotatedRect> &textBoxes, char * title);

/// cardClass: 1 is FROM_TW, 2 is TO_HK
int cardClass = 1;
void cardMode(int cardclass = 1) {
  /// cardClass: 1 is FROM_TW, 2 is TO_HK
  cardClass = cardclass;

  //if (cardClass == 1)
  //  LOGD("from tw/hk");
  //else if (cardClass == 2)
  //  LOGD("to hk");
}

/*************************************************
Function:       tdr_card_scan
Description:    定位卡片位置，定位卡片上的有效字符位置，识别有效字符信息。

Calls:          detect_card: 定位卡片位置，并提供校正后的卡片信息图。
                detectLettersRect_new3: 在卡片信息图正面，定位有效信息行。
                rapidocr_front: 根据正面有效信息行，识别有效信息。
                detectLettersRect_back: 在卡片信息图背面，定位有效信息行。
                rapidocr_back: 根据背面有效信息行，识别有效信息。

Input:          src: 原始彩色图像
                ocr_result: 存储返回的识别结果

Return:         成功返回0， 其他情况返回相应错误状态值。
*************************************************/
int tdr_card_scan(Mat& src, OCR_RESULT& ocr_result) {
#ifdef SUPPORT_TEST_CTL
  append_test_info_item("tdr_card_scan: ", NO_TEST_PARA);
#endif

  double start = tdr_get_current_time();
  int ret = 0;
  Mat dst, card;
  clearCardResult();
  dst = src.clone();

  bool isfront;
  Rect rectFace;
  ret = detect_card(dst, card, isfront, rectFace, isOldCamera());
  if (ret != 0) {
    LOGD("card detect: %d", ret);
    if (ret == ERROR_CARD_LOCATION)
      return ret;
    else
      return ret;//ERROR_LOCATION;
  }

  vector<Rect> lineRect;
  if (isfront) {
//#ifdef USE_CARD_FRONT
    // LOGD("front");
    ret=  check_and_rotated_card_front(card, rectFace, lineRect, cardClass, isOldCamera());
    if(ret!=0) {
         LOGD("front title not found");
         return ret;
    }
/*
    ret=checkFrontValid(card, isOldCamera());
    if(ret!=0) {
         LOGD("front id extract error");
        return ret;
    }
*/



    //ret = detectLettersRect_new3(card, rectFace, lineRect, cardClass, isOldCamera());
    //if (ret != 0) {
      //LOGD("card front detect text: %d", ret);
      //return ERROR_LOCATION;
    //}
//#else
/*
    ret=  check_and_rotated_card_front(card, rectFace, lineRect, cardClass, isOldCamera());
    if (ret != 0) {
      LOGD("card front detect text: %d", ret);
      return ERROR_LOCATION;
    }
*/
//#endif
  } else {
    // LOGD("back");
    ret = detectLettersRect_back(card, lineRect, isOldCamera());
    if (ret != 0) {
      LOGD("card back detect text: %d", ret);
      //return ERROR_LOCATION;
      return ret;
    }
  }

  /// rapidocr
  if (isfront) {
#ifdef USE_CARD_FRONT
    ret = rapidocr_front(card, lineRect, cardClass, isOldCamera());
    if (ret < 0) {
      LOGD("card front scan: %d", ret);
      return ret;
    }
#else
    int x1,x2,y1,y2;
    x1=rectFace.x;
    y1=rectFace.y;
    x2=rectFace.x+rectFace.width-1;
    y2=rectFace.y+rectFace.height-1;
    int center_x= x1+ rectFace.width/2;
    int center_y= y1+ rectFace.height/2;
    float scale_x=1.7;
    float scale_y=1.8;
    int half_width=(float)rectFace.width/2.0*scale_x;
    int half_height=(float)rectFace.height/2.0*scale_y;

    x1=center_x-half_width;
    x2=center_x+half_width;
    y1=center_y-half_height;
    y2=center_y+half_height;
    x1=x1+4; x2=x2+4; y1=y1-2; y2=y2-2;
    if(x1<0) x1=0;
    if(y1<0) y1=0;
    if(x2>=card.cols) x2=card.cols-1;
    if(y2>=card.rows) y2=card.rows-1;
    cv::Rect face_rect;
    face_rect.x=x1;
    face_rect.y=y1;
    face_rect.width=x2-x1+1;
    face_rect.height=y2-y1+1;
    g_faceimg=card(face_rect).clone();
    g_cropimg=card(cv::Rect(10,10, card.cols-20, card.rows-20)).clone();
#ifdef TDR_TRACE
if(isDebugShow())
{
cv::Mat showmat =  card.clone();
rectangle(showmat, rectFace, Scalar(0, 255, 0), 2, LINE_8, 0);
rectangle(showmat, face_rect, Scalar(0, 0, 255), 2, LINE_8, 0);
//imshow("crop", g_cropimg);
imshow("face", showmat);
}
#endif    

#endif
  } else {
    int ret;
    ret = rapidocr_back(card, lineRect, isOldCamera());
    if (ret < 0) {
      LOGD("card back scan: %d", ret);
      return ret;
    }
  }


  if (ret == 0) {
    copy_ocr_result(card_result, ocr_result);
    print_passport_result(ocr_result);
  }

  double end = tdr_get_current_time();
  LOGD("tdr_card_scan cost %.3f ms", end - start);

#ifdef SUPPORT_TEST_CTL
  append_test_info_item("tdr_card_scan: ", (int)(end - start));
#endif
  return ret;
}
