#include <opencv2/opencv.hpp>
#include <iostream>
#include "common.h"
#include <omp.h>
#include "tdrocr.h"
#include "utils.h"
#include "passport_visa.h"

#ifdef SPPORT_MRZ_CHECK
void BGR2GRAY(cv::Mat & input, cv::Mat & output, int color_mode);
int  adjustBright(cv::Mat & gray);

cv::Mat mrz_get_mser_region(Mat& src, Mat & rzimg, float scale, float mser_para1,float mser_para2,float mser_para3,float mser_para4, int retry) {
  int width, height;
  Mat gray;
  width = src.cols / scale;
  height = src.rows / scale;

  if (rzimg.channels() > 1) {
    BGR2GRAY(rzimg, gray, 1);
  } else {
    gray = rzimg;
  }
  cv::GaussianBlur(gray, gray, cv::Size(3, 3), 1, 0);

  int ret=adjustBright(gray);
  if(ret<0) mser_para4=0.8;

  cv::Ptr<cv::MSER> ptrMSER = cv::MSER::create(
      mser_para1, mser_para2, mser_para3, mser_para4); 

  std::vector<std::vector<cv::Point> > points;
  std::vector<cv::Rect> rects;
  ptrMSER->detectRegions(gray, points, rects);

  Mat test_mask = Mat::zeros(rzimg.rows, rzimg.cols, CV_8UC1);

  vector<cv::Rect> charRects;
  for (int i = 0; i < points.size(); i++) {
    Rect brect = boundingRect(points[i]);
    int width = brect.width;
    int height = brect.height;
    if (float(height) / float(width) < 4 &&
        float(height) / float(width) > 0.25) 
    {
      charRects.push_back(brect);
    }
  }

  for (int i = 0; i < charRects.size(); i++) {
    rectangle(test_mask, charRects[i], Scalar(255), -1);
  }
#ifdef TDR_TRACE
// imshow("mrz_mser src", test_mask);
#endif

  resize(test_mask, test_mask, Size(width, height), cv::INTER_NEAREST);
  return test_mask;
}

int mrz_detect_rect(Mat& dispay, Mat& src,
                    int morph_size, float scale )  // default morph_size=20
{
  Mat element = getStructuringElement(MORPH_RECT, Size(morph_size, 1));
  Mat linemask = Mat::zeros(src.rows, src.cols, CV_8UC1);
  morphologyEx(src, linemask, CV_MOP_CLOSE, element);
#ifdef TDR_TRACE
  if (isDebugShow()) {
    imshow("mrz_mser", src);
    imshow("mrz_mser_close", linemask);
     //imwrite("mrz_mser_close.png", linemask);
    // cv::waitKey(0);
  }
#endif

  vector<vector<Point> > contours_mask;
  findContours(linemask, contours_mask, 0, 1);

  vector<RotatedRect> rotatedRect;
  ;
  vector<vector<Point> > contours_poly(contours_mask.size());
  for (size_t i = 0; i < contours_mask.size(); i++) {
    if (contours_mask[i].size() > 100) {
      approxPolyDP(Mat(contours_mask[i]), contours_poly[i], 0, true);
      Rect appRect(boundingRect(Mat(contours_poly[i])));
      if (appRect.width > appRect.height) {
        rotatedRect.push_back(minAreaRect(contours_poly[i]));
      }
    }
  }
  //LOGD("valid rotatedRect.size(): %d", rotatedRect.size());
  if (rotatedRect.size() <= 0) return -1;

  int max_width_idx = -1;
  float max_width = 0;
  for (int i = 0; i < rotatedRect.size(); i++) {
    // TODO 添加有效字符行的筛选条件 例如
    float h = min (rotatedRect[i].size.height, rotatedRect[i].size.width);
    float w = max (rotatedRect[i].size.height, rotatedRect[i].size.width);
    //LOGD("w=%f, h=%f", w, h);
    //if (h<25 && h > 15 && w > 558 && w<685) // TODO d区的图可能大部分是1代，阈值可能会大
    //if (h<22 && h > 10 && w > 450 && w<590) 
    if(h<120 && h>13 && w> 440 )
    {
      if (max_width < w || max_width == 0) {
        max_width = w;
        max_width_idx = i;
      }
    }
  }

//cv::waitKey(0);
//return 0;

  if (max_width_idx == -1) return -2;
  //LOGD("max_width_idx: %d", max_width_idx);

#ifdef TDR_TRACE
  if (isDebugShow()) {
    //Mat test_mrz = dispay;
    //rotatedRect[max_width_idx].center.x*=scale;
    //rotatedRect[max_width_idx].center.y*=scale;
    //circle(test_mrz, rotatedRect[max_width_idx].center, 4, Scalar(0, 0, 255));
    //imshow("test_mrz", test_mrz);
  }
#endif

  LOGD("max_length_mrz_center: %.2f, %.2f", rotatedRect[max_width_idx].center.x, rotatedRect[max_width_idx].center.y);
  if(isMoved(0, rotatedRect[max_width_idx].center.x, rotatedRect[max_width_idx].center.y)<0) return ERROR_NEXT;

  return 0;
}

int check_mrz_main(Mat& src, Mat& mserMask) {
    int ret = 0;
    int width, height;
    Mat rzimg, gray;
    float   scale = src.cols / 600;
    width = src.cols / scale;
    height = src.rows / scale;

    int rz_width = width * 0.75;
    int rz_height = height * 0.75;

    resize(src, rzimg, Size(rz_width, rz_height), cv::INTER_NEAREST);

    cv::Mat this_mserMask = mrz_get_mser_region(src, rzimg, scale, 5, 8, 100 , 0.5, 0);
    //ret = mrz_detect_rect(src, this_mserMask, 20, scale);
    ret = mrz_detect_rect(src, this_mserMask, 20, scale);

    if(ret != 0) 
    {
        //LOGD("retry");
        //this_mserMask = mrz_get_mser_region(src, rzimg, scale, 5, 8, 100 , 0.5, 1);
        //ret = mrz_detect_rect(src, this_mserMask, 20, scale);
    } 
    if(ret==0) mserMask=this_mserMask.clone();
    return ret;
}

#endif
