#include <opencv2/opencv.hpp>
#include <iostream>
#include "common.h"
#include <omp.h>
#include "tdrocr.h"
#include "utils.h"
#include "passport_visa.h"

int char_segment(int iscard, cv::Mat &line1, cv::Mat &line2, cv::Mat &line1_rgb, cv::Mat &line2_rgb, int &sz1, int &sz2);
void BGR2GRAY(cv::Mat & input, cv::Mat & output, int color_mode);
bool rotation_visa2(Mat &in, Mat &out, Size rect_size, Point2f center, const double angle, PASSPORT_ROTATED_STRUCT &rotated_struct, bool set_rotated_struct);
int detectMrz(Mat& src, vector<RotatedRect> & rr);
int charDtct2(Mat& src, Mat& line1_rgb, Mat& line2_rgb, Mat& line1, Mat& line2, int &sz1, int &sz2);
int charDtct(Mat& src, Mat& mserMask, Mat& line1_rgb, Mat& line2_rgb, Mat& line1, Mat& line2, int &sz1, int &sz2);
extern PASSPORT_ROTATED_STRUCT g_rotated_struct[2];
Mat Sauvola_coarse(Mat inpImg, int window_half_length, float k, float super_std=20);
void gamma_correction(unsigned char * src,int width,int height,float gamma);
int  getBinThreshold(cv::Mat & src);
extern int g_detect_mode;
extern int g_set_detect_mode;
extern int g_scan_none;


int passport_location(cv::Mat& src, int &sz1, int &sz2)
{
        double start=tdr_get_current_time();
        #ifdef SUPPORT_VISA
        clearVisaIssueInfo();
        #endif

        if(g_detect_mode==2){
             CUT_SIZE=480;
        }
        else{
             CUT_SIZE=480;
        }

        Mat roi = src(Rect(0, src.rows-CUT_SIZE-1,  src.cols, CUT_SIZE));
        cv::Mat line1_rgb, line2_rgb, line1, line2;
        int ret;

        //////////////////////////////////
        #ifdef SPPORT_MRZ_CHECK
        extern int check_mrz_main(Mat& src, Mat &masrMask) ;
        //cout<<"##############SPPORT_MRZ_CHECK################"<<endl;
        cv::Mat mserMask;
        ret =  check_mrz_main(roi, mserMask);
        LOGD("mrz check: %d", ret);
        if(ret != 0)
            return ret;
        #endif
        /////////////////////////////////

        if(g_scan_none==1) return 0;

        if(g_set_detect_mode>0) g_detect_mode=g_set_detect_mode-1;

        LOGD("detect_mode: %d, cut_size: %d", g_detect_mode, CUT_SIZE);
        if(g_detect_mode==0){
             ret = charDtct(roi, mserMask, line1_rgb, line2_rgb, line1, line2, sz1, sz2);
        }
        else{
             ret = charDtct2(roi, line1_rgb, line2_rgb, line1, line2, sz1, sz2);
        }

        g_detect_mode++;
        if(g_detect_mode>1) g_detect_mode=0;

        double end=tdr_get_current_time();
        LOGD("locat and split char cost %.3f ms", end-start);

        return ret;
}

int g_swap_detect=0;
int charDtct2(Mat& src, Mat& line1_rgb, Mat& line2_rgb, Mat& line1_bin, Mat& line2_bin, int &sz1, int &sz2)
{
      int scale, width, height;
      Mat resizeimg;
      Mat line1_gray, line2_gray;

      if(src.cols < 600) {
            showError(ERROR_IMAGE_SIZE);
            return ERROR_IMAGE_SIZE;
      }

      scale=src.cols/600;
      float scale_x= scale;
      float scale_y= scale;
      width = src.cols/scale_x;
      height = src.rows/scale_y;
      if( src.cols<=0 || src.rows<=0 || width<=0 || height<=0 ) return -1;
      vector<RotatedRect>  rr;
      int ret=detectMrz(src, rr);
      if(ret<0) return -1;

      int color_mode;
      int threshold1, threshold2;
         color_mode=0;
         threshold1=31;
         threshold2=17;
         g_swap_detect=0;
      bool rets[2]; rets[0]=false; rets[1]=false;
      LOGD("color_mode:%d, threshold:%d,%d", color_mode, threshold1, threshold2);

//#ifdef CHECK_MOVED
     //if(isMoved(0, rr[0].center.x, rr[0].center.y)<0) return ERROR_NEXT;
//#endif

      cv::Mat small[2];
      Point2f new_center[2];
      Rect2f safe_rect[2];
      for(int i=0; i<2; i++)
      {
            if (rr[i].angle < -45) rr[i].angle += 90;
            if(rr[i].size.height>rr[i].size.width){
                swap(rr[i].size.height, rr[i].size.width);
            }
            bool isFormRect = calcSafeRect(rr[i], src, safe_rect[i]);
            small[i] = src(safe_rect[i]);
            new_center[i] = rr[i].center - cv::Point2f(safe_rect[i].x, safe_rect[i].y);
      }

//#ifndef TDR_TRACE
#pragma omp for
//#endif
      for(int i=0; i<2; i++)
      {
        if (i == 0){
            rets[i] = rotation_visa2(small[i], line1_rgb, rr[i].size, new_center[i], rr[i].angle, g_rotated_struct[i], true);
            g_rotated_struct[i].rrect = safe_rect[i];
        }
        else
            rets[i] = rotation_visa2(small[i], line2_rgb, rr[i].size, new_center[i], rr[i].angle, g_rotated_struct[i], false);

            if(rets[i])
            {
                if(i==0) BGR2GRAY(line1_rgb, line1_gray, color_mode);
                else     BGR2GRAY(line2_rgb, line2_gray, color_mode);
            }
      }


      if(!rets[0] || !rets[1]) return -1;
     
      //threshold2 = getBinThreshold(line1_gray);
      preprocess1(line1_gray, line2_gray, line1_bin, line2_bin, threshold1, threshold2);
      ret=char_segment(0, line1_bin, line2_bin, line1_rgb, line2_rgb, sz1, sz2);
      if(ret!=0)
      {
           if(sz1>44 || sz2>44){
              LOGD("try threshold 25");
              preprocess1(line1_gray, line2_gray, line1_bin, line2_bin, 31, 25);
              ret=char_segment(0, line1_bin, line2_bin, line1_rgb, line2_rgb, sz1, sz2);

              if(ret!=0){
              LOGD("try threshold 11");
              preprocess1(line1_gray, line2_gray, line1_bin, line2_bin, 31, 11);
              ret=char_segment(0, line1_bin, line2_bin, line1_rgb, line2_rgb, sz1, sz2);
              }
           }
           else{
              LOGD("try threshold 11");
              preprocess1(line1_gray, line2_gray, line1_bin, line2_bin, 31, 11); 
              ret=char_segment(0, line1_bin, line2_bin, line1_rgb, line2_rgb, sz1, sz2);

              if(ret!=0){
              LOGD("try threshold 25");
              preprocess1(line1_gray, line2_gray, line1_bin, line2_bin, 31, 25);
              ret=char_segment(0, line1_bin, line2_bin, line1_rgb, line2_rgb, sz1, sz2);
              }
           }
      }
     
      return ret;
}


Mat drawRotatedRect(Mat &in, Size rect_size, Point2f center, const double angle)
{
    // 创建旋转矩形
    RotatedRect rotatedRect(center, rect_size, angle);
    Mat out = in.clone();
    // 获取旋转矩形的四个顶点
    Point2f vertices[4];
    rotatedRect.points(vertices);

    // 将顶点坐标转换为整数
    for (int i = 0; i < 4; i++)
    {
        line(out, vertices[i], vertices[(i + 1) % 4], Scalar(0, 255, 0), 2);
    }
    if (angle < 0)
    {
        circle(out, vertices[0], 2, Scalar(0, 0, 255), -1);
    }
    else
    {
        circle(out, vertices[3], 2, Scalar(0, 0, 255), -1);
    }
    return out;
}


/**
 * @brief mrz区域旋转矫正函数，返回矫正后的一行line
 *
 * @param in            输入旋转前的mrz区域
 * @param out           输出一行line
 * @param rect_size     一行line的size
 * @param center        line的中心
 * @param angle         line的旋转角度
 * @param rotated_struct
 * @param set_rotated_struct
 * @return true         成功
 * @return false        输入图像或rect_size不正确
 */
bool rotation_visa2(Mat &in, Mat &out, Size rect_size, Point2f center, const double angle,
                   PASSPORT_ROTATED_STRUCT &rotated_struct, bool set_rotated_struct)
{ // set_rotated_struct 用于核并行保护

    if (in.empty())
    {
        return false;
    }
    if (rect_size.width <= 0 || rect_size.height <= 0)
    {
        return false;
    }

    Mat in_large;
    int w = in.cols, h = in.rows;
    // 求rect的y值最大的点作为旋转点
    RotatedRect rotatedRect(center, rect_size, angle);
    Point2f vertices[4];
    rotatedRect.points(vertices);

    Point2f rotate_center; // 旋转的中心点
    int sw = rect_size.width;
    int sh = rect_size.height;
    // cout << "w: " << w << " h: " << h << endl;
    // cout << "sw: " << sw << " sh: " << sh << endl;
    // cout << "cent: " << center << endl;
    // cout << "angle: " << angle << endl;
    if (angle < 0)
    {
        rotate_center = vertices[0];
    }
    else
    {
        rotate_center = vertices[3];
    }
    Mat rot_mat;
    double _angle = angle;
    // 根据rect中y值最大的角点的 最邻近的两个字符是否越出图像底边 判定是否旋转
    double char_point_x;
    double char_point_y;
    // 计算出邻近的两个字符的最大的y值
    if (angle < 0)
    {
        rotate_center = vertices[0];
        char_point_x = rotate_center.x + 45;
    }
    else
    {
        rotate_center = vertices[3];
        char_point_x = rotate_center.x - 45;
    }
    float slope = (vertices[3].y - vertices[0].y) / (vertices[3].x - vertices[0].x);
    char_point_y = slope * (char_point_x - rotate_center.x) + rotate_center.y;

    Point2f new_center;

    // 修改1：角度阈值、裁切方式
    // 不旋转的情况，直接切出不越界的外接矩形区域
    if ((abs(angle) < 1.0 && h - char_point_y < 1) || abs(angle) < 0.2)
    {
        // cout << "不旋转" << endl;
        out = in;
        in_large = in;
        rot_mat = getRotationMatrix2D(rotate_center, 0.0, 1.0);
        _angle = 0;
        // 找到外接矩形的左上角的点
        Point2f tl;
        tl.x = std::min({vertices[3].x, vertices[0].x, vertices[1].x, vertices[2].x});
        tl.y = std::min({vertices[3].y, vertices[0].y, vertices[1].y, vertices[2].y});
        rect_size.width = abs(vertices[2].x - vertices[0].x);
        rect_size.height = abs(vertices[2].y - vertices[0].y);
        // 如果左、下越界则以图像边缘为边界
        if (tl.x < 0)
        {
            tl.x = 0;
            rect_size.width = w + tl.x;
        }
        if (rotate_center.y > h)
        {
            rect_size.height = h - tl.y;
        }
        new_center.x = tl.x + rect_size.width / 2;
        new_center.y = tl.y + rect_size.height / 2;
    }
    // 旋转的情况
    else
    {
        // 计算旋转矩阵
        rot_mat = getRotationMatrix2D(rotate_center, angle, 1.0);
        // cout << "旋转" << endl;
        // 计算旋转后四个角的位置
        vector<Point2f> corners(4);
        corners[0] = Point2f(0, 0);
        corners[1] = Point2f(in.cols-1, 0);
        corners[2] = Point2f(in.cols-1, in.rows-1);
        corners[3] = Point2f(0, in.rows-1);
        Mat cornersMat(corners);
        transform(cornersMat, cornersMat, rot_mat);

        // 找到旋转后的边界
        Rect boundingBox = boundingRect(cornersMat);
        rot_mat.at<double>(0, 2) -= boundingBox.x; // 调整X轴平移
        rot_mat.at<double>(1, 2) -= boundingBox.y; // 调整Y轴平移

        // 旋转变化
        warpAffine(in, in_large, rot_mat, boundingBox.size(), INTER_LINEAR, BORDER_REPLICATE);   // 修改2：填充方式

        // 求line的中心点在旋转后图像的坐标
        Mat p_mat = (Mat_<double>(3, 1) << center.x, center.y, 1);
        Mat rotated_p_mat = rot_mat * p_mat;
        new_center.x = rotated_p_mat.at<double>(0, 0);
        new_center.y = rotated_p_mat.at<double>(1, 0);

        // 当旋转角度较大，并且字符越过图像底部时，需要将填充区域切割掉
        if (abs(angle) > 1.5 && h - char_point_y < -7)
        {
            // cout << "大角度" << endl;
            double offset = rotate_center.y - h;
            // 如果rect越过图像底部不超过大约0.4个大小，裁掉填充区域不会对字符完整性造成太大影响
            if (offset < rect_size.height * 0.4)
            {
                rect_size.height = rect_size.height - offset + 10;
                new_center.y = new_center.y - 0.5 * offset + 2;
            }
            // 如果rect越过图像底部超过0.4个大小，字符已经不完整了，只能尽量去除白边，尽量保证字符完整
            else
            {
                rect_size.height -= 10;
                new_center.y -= 4;
            }
        }
    }
        // 裁剪出line区域
        Mat img_crop;
        getRectSubPix(in_large, Size(rect_size.width, rect_size.height), new_center, img_crop);
        out = img_crop;


#ifdef SUPPORT_VISA
    if (true == set_rotated_struct && false == rotated_struct.im_valid)
    {
        // double start=tdr_get_current_time();
        invertAffineTransform(rot_mat, rotated_struct.im);
        rotated_struct.before_affine_cut_diff = Size(0, 0);
        rotated_struct.after_affine_cut_diff = Size((in_large.cols - rect_size.width) / 2, (in_large.rows - rect_size.height) / 2);
        rotated_struct.roi_angle = _angle;
        rotated_struct.im_valid = true;
        // double end=tdr_get_current_time();
        // LOGD("rotation_visa + stemp cost %.3f ms", end-start);
    }
#endif
    return true;
}
